#!/usr/local/bin/python

# $Id: chanfix.py,v 1.6 2002/08/20 07:05:41 dbt Exp $

__doc__ = '''
Settings must be specified as follows (in a file called config.chanfix):
addr = (host,port)
spoofnick = 'fakenick'
spoofircname = 'the irc name and server name for spoofing'
spoofusername = 'username of spoofed user'
spoofhostname = 'hostname of spoofed user'
spoofserver = 'servername of spoofed server'
serverpass = 'password'
opnick = 'nickname to op'
oponchans = None (or a list/tuple of channels)

spoofserver and serverpass must match an N: line on 
the server listening on addr.

opnick and oponchans can be overridden on the command line, 
by presenting the nick and then as many channels as you want
the user to be opped on.
'''

import sys, socket, asyncore, asynchat, time

cfg = {}

execfile('config.chanfix',cfg)

defopts = ('addr','spoofnick','spoofircname','spoofhostname','opnick',
           'spoofserver','serverpass','oponchans','spoofusername')

defargs = {}
for k in defopts:
    if cfg.has_key(k):
        defargs[k] = cfg[k]
    else:
        print "Missing option: %s\n" % k



class Fixer(asynchat.async_chat):
    def __init__(self, *args, **kwargs):
        self.chans = {}
        self.postping = ''
        self.settings = {}
        for k in kwargs.keys():
            if defargs.has_key(k):
                self.settings[k] = kwargs[k]
            else:
                raise AttributeError, "invalid option: " + k
        for k in defargs.keys():
            if not self.settings.has_key(k):
                self.settings[k] = defargs[k]
        asynchat.async_chat.__init__(self)
        self.buf = ''
        self.terminator = '\r\n'
        self.settings['matches'] = (self.settings['opnick'].lower(), '+'+self.settings['opnick'].lower())

    def collect_incoming_data(self, data):
        self.buf += data

    def found_terminator(self):
        t = self.buf.split(' :',1)
        if len(t) == 2:
            verbs,data = t
        else:
            data = ''
            verbs = self.buf
        verbs = verbs.split()
        prefix = None
        if verbs[0][0] == ':':
            prefix = verbs.pop(0)
        cmd = verbs.pop(0)
        data = verbs + [data]
        meth = "do_" + cmd.lower()
        try:
            if cmd == str(int(cmd)):
                print self.buf
        except:
            pass
        if hasattr(self,meth):
            getattr(self, meth)(prefix=prefix, cmd=cmd, args=data)
        self.buf = ''

    def do_sjoin(self,prefix,cmd,args):
        nicks = args.pop()
        ts = args.pop(0)
        chan = args.pop(0)
        mode = ' '.join(args)
        d = {}
        self.chans[chan] = d
        d['mode'] = mode
        n = []
        d['nicks'] = n
	matches = self.settings['matches']
        for ni in nicks.split():
            n.append(ni)
            if ni.lower() in matches:
                self.push(':%s SJOIN %s %s %s :@%s\r\n' % (self.settings['spoofserver'], ts, chan, mode, self.settings['spoofnick']))
                self.postping += ':%s MODE %s +o %s\r\n' % (self.settings['spoofnick'], chan, matches[0])
                pass

    def do_nick(self,prefix,cmd,args):
        # ['NICK', 'dbt', '1', '1029804873', '+iw', 'dbt', '127.0.0.1', 'laptop.catch22.org', "Karl Marx hasn't seen anything yet"]
        # NICK dbt 1 1029804873 +iw dbt 127.0.0.1 laptop.catch22.org :Karl Marx hasn't seen anything yet
        nick, hop, ts, mode, user, host, server, ircname = args

    def do_ping(self,prefix,cmd,args):
        self.push('PONG :%s\r\n' % args[-1])
        self.push(self.postping)
        self.push('SQUIT\r\n')

    def handle_connect(self):
        self.push('PASS %(serverpass)s :TS\r\nCAPAB :QS EX CHW KNOCK\r\nSERVER %(spoofserver)s 1 :%(spoofircname)s\r\n' % self.settings)
        self.settings['now'] = str(int(time.time()))
        self.push('NICK %(spoofnick)s 1 %(now)s +i %(spoofusername)s %(spoofhostname)s %(spoofserver)s :%(spoofircname)s\r\n' % self.settings)

        
    def run(self):
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connect(self.settings['addr'])
        asyncore.loop()
        

if __name__ == '__main__':
    if len(sys.argv) == 2:
        f = Fixer(opnick = sys.argv[1])
    elif len(sys.argv) > 2:
        f = Fixer(opnick = sys.argv[1], oponchans = sys.argv[2:])
    else:
        f = Fixer()
    f.run()

